/*
 * @(#) ControlResourceImpl.java
 *
 * JOTM: Java Open Transaction Manager 
 *
 * This project is developed inside the ObjectWeb Consortium,
 * http://www.objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ControlResourceImpl.java,v 1.8 2003/03/24 13:13:34 jmesnil Exp $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jotm;

import org.omg.CosTransactions._ResourceImplBase;
import org.omg.CosTransactions.Vote;
import org.omg.CosTransactions.NotPrepared;
import org.omg.CosTransactions.HeuristicRollback;
import org.omg.CosTransactions.HeuristicMixed;
import org.omg.CosTransactions.HeuristicCommit;
import org.omg.CosTransactions.HeuristicHazard;
import org.omg.CORBA.UNKNOWN;
import org.omg.CORBA.TRANSACTION_ROLLEDBACK;

import java.rmi.RemoteException;

/**
 * This class wraps a JOnAS ControlImpl as a CORBA Resource, allowing it to be registered
 * by a CORBA supcoord.
 */
public class ControlResourceImpl extends _ResourceImplBase {

    /**
     * the wrapped ControlImpl
     * @serial
     */
    private ControlImpl myctrl;

    /**
     * is true only if prepared, but not yet committed or rolled back.
     * @serial
     */
    private boolean prepared = false;


    // ------------------------------------------------------------------
    // Constructors
    // ------------------------------------------------------------------

    /**
     * constructor used by ControlImpl when registered in its supcoord.
     */
    ControlResourceImpl(ControlImpl ctrl) {
	TraceTm.jotm.debug("ControlResourceImpl.ControlResourceImpl(ControlImpl)");
	this.myctrl = ctrl;
    }

    // ------------------------------------------------------------------
    // Resource interface
    // ------------------------------------------------------------------

    /** 
     * phase 1 of the 2PC.
     *
     * @return int vote commit, rollback, or readonly.
     */
    public Vote prepare() throws HeuristicMixed, HeuristicHazard  {
	TraceTm.jotm.debug("ControlResourceImpl.prepare()");
	try {
	    Vote ret;
	    switch (myctrl.prepare()) {
	    case org.objectweb.jotm.Resource.VOTE_COMMIT:
		ret = Vote.VoteCommit;
	    case org.objectweb.jotm.Resource.VOTE_ROLLBACK:
		ret = Vote.VoteRollback;
	    case org.objectweb.jotm.Resource.VOTE_READONLY:
		ret = Vote.VoteReadOnly;
	    default:
		//unexpected
		TraceTm.jotm.error("ControlResourceImpl: unknown Vote value on prepare");
		ret = Vote.VoteReadOnly;
	    }
	    prepared = true;
	    return ret;
	} catch (org.objectweb.jotm.HeuristicMixed e) {
	    throw new HeuristicMixed();
	} catch (RemoteException e) {
	    throw new UNKNOWN("JOnAS TM RemoteException: " + e.getMessage());
	}
    }

    /** 
     * rollback transaction
     */
    public void rollback() throws HeuristicMixed, HeuristicHazard, HeuristicCommit {
	TraceTm.jotm.debug("ControlResourceImpl.rollback()");
	try {
	    myctrl.rollback();
	    prepared = false;
	} catch (org.objectweb.jotm.HeuristicMixed e) {
	    throw new HeuristicMixed();
	} catch (RemoteException e) {
	    throw new UNKNOWN("JOnAS TM RemoteException: " + e.getMessage());
	}
    }

    /** 
     * phase 2 of the 2PC.
     */
    public void commit() throws NotPrepared,HeuristicRollback,HeuristicMixed,HeuristicHazard {
	TraceTm.jotm.debug("ControlResourceImpl.commit()");
	try {
	    if (!prepared)
		throw new NotPrepared();
	    myctrl.commit();
	    prepared = false;
	} catch (org.objectweb.jotm.HeuristicMixed e) {
	    throw new HeuristicMixed();
	} catch (org.objectweb.jotm.HeuristicHazard e) {
	    throw new HeuristicHazard();
	} catch (javax.transaction.TransactionRolledbackException e) {
	    throw new TRANSACTION_ROLLEDBACK();
	} catch (RemoteException e) {
	    throw new UNKNOWN("JOnAS TM RemoteException: " + e.getMessage());
	}
    }


    /** 
     * commit 1 phase.
     */
    public void commit_one_phase() throws HeuristicHazard {
	TraceTm.jotm.debug("ControlResourceImpl.commit_one_phase()");
	try {
	    myctrl.commit_one_phase();
	//} catch (org.objectweb.jotm.HeuristicMixed e) {
	//    throw new HeuristicMixed();
	} catch (org.objectweb.jotm.HeuristicHazard e) {
	    throw new HeuristicHazard();
	} catch (javax.transaction.TransactionRolledbackException e) {
	    throw new TRANSACTION_ROLLEDBACK();
	} catch (RemoteException e) {
	    throw new UNKNOWN("JOnAS TM RemoteException: " + e.getMessage());
	}
    }

    /**
     * forget heuristics about this transaction.
     */
    public void forget() {
	TraceTm.jotm.debug("ControlResourceImpl.forget()");
	try {
	    myctrl.forget();
	} catch (RemoteException e) {
	    throw new UNKNOWN("JOnAS TM RemoteException: " + e.getMessage());
	}
    }

}
